# -*- coding: UTF-8 -*-

from nes_common.inserter import Inserter 
from cs.mysql.nes_exec import Exec
import MySQLdb
import ConfigParser
import sys
from nes_common import logger
from nes_command import Command

class DB_Inserter(Inserter):

    dbVersionIsOK = {}

    def __init__(self, shop, loadDictStatusSender=None):
        log = logger.Logger().getLogger(shop.get('shopId', ''))
        config = ConfigParser.RawConfigParser()
        if not config.read(sys.path[0] + "/../../../config/nes_load.ini"):
            log.error("Configuration file 'nes_load.ini' not found.")
            sys.exit(1)
        host = config.get('mysql', 'host')
        port = config.getint('mysql', 'port')
        user = config.get('mysql', 'user')
        password = config.get('mysql', 'password')
        self.migrateVersion = config.getint('mysql', 'migrateVersion')
        notBlockDBForCommandsStr = config.get('MAIN', 'notBlockDBForCommands') if config.has_section('MAIN') and config.has_option('MAIN', 'notBlockDBForCommands') else ''
        notBlockDBForCommands = [s.strip() for s in notBlockDBForCommandsStr.split(',')]
        suffix = shop.get('shopId', '')
        self.modes = ['ANY','CS']
        if shop.get('shopId', '').endswith('All') and shop.get('shopId', '').startswith(shop.get('clusterId', '')):
            suffix = 'All'
            self.modes.append('CS_ALL')
        executor = Exec(host=host, port=port, user=user, password=password, database='%sdictionaries%s' % (shop.get('clusterId', ''), suffix), shopCode=shop.get('shopId', ''), config=config, loadDictStatusSender=loadDictStatusSender)
        Inserter.__init__(self, shop.get('shopId', ''), executor, notBlockDBForCommands)
        self.mysqlConnBlock = MySQLdb.connect(host=host, user=user, passwd=password, db='artixcs_struct', port=port)
        self.mysqlConnBlock.autocommit(True)
        self.dbForCheckVersion = '%sartixcs%s' % (shop.get('clusterId', ''), suffix)
        self.mysqlConnDBVersion = MySQLdb.connect(host=host, user=user, passwd=password, db=self.dbForCheckVersion, port=port)
        self.mysqlConnDBVersion.autocommit(True)
        if not self.dbForCheckVersion in self.dbVersionIsOK:
            self.dbVersionIsOK[self.dbForCheckVersion] = False

    def testDBVersion(self):
        if self.dbVersionIsOK.get(self.dbForCheckVersion, False):
            return True
        try:
            cur = self.mysqlConnDBVersion.cursor()
            cur.execute("select version from migrate_version where repository_id like 'artixcs%'")
            row = cur.fetchone()
            dbVersion = row[0] if row != None else self.migrateVersion
            self.log.debug('migrateVersion fom DB = %s' % dbVersion)
            self.log.debug('migrateVersion fom config = %s' % self.migrateVersion)
            if dbVersion >= self.migrateVersion:
                self.dbVersionIsOK[self.dbForCheckVersion] = True
                return True
        except Exception, e:
            self.log.warning('%s' % e)
            return False
        finally:
            cur.close()
        return False

    def flagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('Block storeId=%s' % self.shopCode)
        try:
            res = cur.execute("insert into extlocktable (storeId,lockedBy) values ( '%s', 'NES')" % self.shopCode)
            if res == 1:
                self.blocked = True
                return 1
        except:
            return 0
        finally:
            cur.close()
        return 0

    def testFlagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        cur.execute("select * from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        if cur.fetchone() != None:
            cur.close()
            return True
        cur.close()
        return False

    def flagUnBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('UnBlock storeId=%s' % self.shopCode)
        cur.execute("delete from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        cur.close()
        self.blocked = False
        if self.testFlagBlockSet():
            return 0
        return 1
    
    def getCommand(self, simpleCommands):
        return Command(self.executor, self.shopCode, simpleCommands = simpleCommands, modes=self.modes)

