# -*- coding: UTF-8 -*-
'''
Created on 14.04.2010

@author: gsu
'''

from nes_common.sql_operation import _Insert, _Update as Update, _LastInsertId as LastInsertId, \
    _Select as Select, _Delete as Delete, _DeleteAll, _DeleteByDate as DeleteByDate, \
    _DeleteOverduePrices as DeleteOverduePrices, _DeleteBeforeDate as DeleteBeforeDate, \
    _DeleteNotNestedItems
import MySQLdb

class Insert(_Insert):

    def execute(self, context):
        self.filterColumns(self.values, context)
        try:
            _Insert.execute(self, context)
        except MySQLdb.IntegrityError, e:
            if e[0] != 1062:  # пропускаем ошибку mysql (Duplicate entry)
                raise e

    def prepareStatement(self, context):
        if self.cacheKey in self.cache:
            sql = _Insert.cache[self.cacheKey]
        else:
            context.tableName(self.table)  # вот в этом месте кэшируется имя таблицы для дальнейшего определения задачи на выгрузку. Нельзя ее не вызывать
            fields = ['`%s`' % k for k in self.values.keys()]
            if self.quotedValues != None:
                fields.append(self.quotedValues.keys)
            fieldsCount = len(fields)
            dupValue = ",".join(map(lambda f: f + " = values(" + f + ")", fields)) + (
                ',`ownerStoreId` = NULL' if self.cacheOwnerStoreIdPresented.get(self.table, False) else '')
            sql = 'insert ignore into %(table)s (%(fields)s) values (%(values)s) on duplicate key update %(dup)s' % {"table": self.table,
                                                                                                                     "fields": ",".join(fields),
                                                                                                                     "dup": dupValue, "values": ','.join(
                    [self.replaceSymbol] * fieldsCount)}
            _Insert.cache[self.cacheKey] = sql
            _Insert.cacheValues[self.cacheKey] = []
        return sql

class DeleteAll(_DeleteAll):
    def execute(self, context):
        tableName = context.tableName(self.table)
        context.cursor.execute("SET @CLEAR_COMMAND=TRUE;")
        sql = "DELETE FROM %s" % tableName
        context.cursor.execute(sql)
        context.cursor.execute("SET @CLEAR_COMMAND=NULL;")
        context.emptyTables[self.table] = True


class DeleteNotNestedItems(Delete):
    def __init__(self, table, mainCodeField, mainCodeValue, nestedCodeField, nestedCodeIn, replaceSymbol='%s', notExecMany=False):
        self.table = table
        self.mainCodeField = mainCodeField
        self.mainCodeValue = mainCodeValue
        self.nestedCodeField = nestedCodeField
        self.nestedCodeIn = nestedCodeIn if nestedCodeIn else []
        self.whereValues = [mainCodeValue] + nestedCodeIn
        self.replaceSymbol = replaceSymbol
        self.notExecMany = notExecMany

    def __str__(self):
        return '[DELETE table=%s, mainCodeField=%s, mainCodeValue=%s, nestedCodeField=%s, nestedCodeIn=%s]' % (
            self.table, self.mainCodeField, self.mainCodeValue, self.nestedCodeField, self.nestedCodeIn)

    def prepareStatement(self, context):
        hashkey = self.table + '@' + repr(self.mainCodeField) + '_for_' + str(len(self.nestedCodeIn)) + '_' + repr(self.mainCodeValue)
        if hashkey in self.cache:
            sql = self.cache[hashkey]
        else:
            tableName = context.tableName(self.table)
            mainCodeClause = self.mainCodeField + '=' + self.replaceSymbol
            notInClause = ''
            if self.nestedCodeIn:
                notInClause = ' and %s not in (%s)' % (self.nestedCodeField, ','.join([self.replaceSymbol] * len(self.nestedCodeIn)))
            sql = 'delete from %s where %s %s' % (tableName, mainCodeClause, notInClause)
            self.cache[hashkey] = sql
            self.cacheValues[hashkey] = []
        return sql, hashkey
