# -*- coding: UTF-8 -*-
'''
Created on 15.04.2010

@author: gsu
'''

import sys
from nes_common import logger
import ConfigParser
from nes_common import sql_operation
import json
import os
import shutil

def getBoolFromConfig(config, nameOfParam, defaultValue, section):
    return config.getboolean(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

class DataIntegrityError:
    pass

class Executor(object):

    context = None

    def __init__(self, shopCode, connection, context, commandCommitCount):
        self.log = logger.Logger().getLogger(shopCode)
        config = ConfigParser.RawConfigParser()
        if not config.read(sys.path[0] + "/../../../config/nes_load.ini"):
            self.log.error("Configuration file 'nes_load.ini' not found.")

        self.connection = connection
        self.context = context
        self.trickyCounter = 0
        self.commandCommitCount = commandCommitCount
        self.sqlInsert = None
        self.sqlDelete = None

        self.testProgressMode = getBoolFromConfig(config, "testProgressMode", False, "MAIN")
        self.progressStep = 0          # Шаг обновления прогресса
        self.nextStep = 0              # Следующий шаг обновления прогресса
        self.currentObjectsCount = 0   # Объектов загружено
        self.progressEnable = False    # Использовать сохранение прогресса
        self.nesProgressFile = "/linuxcash/cash/exchangesystems/progress/nes.json"
        self.countProgressFile = ''

    def startTransaction(self):
        cur = self.connection.cursor()
        cur.execute('START TRANSACTION;')

    def execute(self, sql):
        try:
            sql.execute(self.context)
        except DataIntegrityError:
            self.log.error('data integrity error. exiting')
            sys.exit(1)

    def executeMany(self):
        if self.context.executeMany:
            if self.sqlDelete:
                self.sqlDelete.execMany(self.context)
            if self.sqlInsert:
                self.sqlInsert.execMany(self.context)

    def _execFinalCheckRemoveRecords(self):
        pass

    def commit(self):
        if self.trickyCounter >= self.commandCommitCount:
            self.executeMany()
            self._execFinalCheckRemoveRecords()
            self.connection.commit()
            self.trickyCounter = 0
        self.trickyCounter += 1

    def rollback(self):
        self.connection.rollback()

    def flushTables(self):
        cur = self.connection.cursor()
        cur.execute('FLUSH tables;')

    def startUpload(self, messageId):
        self.log.info('start upload')
        try:
            sql_operation._Insert.cache.clear()
            sql_operation._Update.cache.clear()
            sql_operation._Select.cache.clear()
            sql_operation._Delete.cache.clear()
        except Exception, e:
            self.log.error('Error clear cache. %s' % e)

        self.flushTables()
        self.initProgress(messageId)

    def finishUpload(self):
        self.log.info(u'Завершение загрузки справочника')
        self.executeMany()
        if self.context.finalize() == False:
            self.log.info('data inegrity error. exiting')
            sys.exit(1)
        self._execFinalCheckRemoveRecords()
        self.connection.commit()
        self.log.info(u"Сброс кэша запросов")
        self.flushTables()
        self.finishProgress()

    '''
    Обновление прогресса загрузки
    '''
    def updateProgress(self):
        if self.progressEnable:
            self.currentObjectsCount += 1
            if self.currentObjectsCount >= self.nextStep:
                self.nextStep += self.progressStep
                percent = int(5 * self.currentObjectsCount / self.progressStep)
                self.log.info(u'%d%% выполнено' % percent)
                self.saveProgress(percent)

    '''
    Сохраняет текущий прогресс в файл
    '''
    def saveProgress(self, value):
        data = {"progress": value}
        try:
            with open(self.nesProgressFile, 'w') as outfile:
                json.dump(data, outfile)
            if self.testProgressMode:
                if not os.path.exists("/tmp/progress"):
                    os.makedirs("/tmp/progress")
                shutil.copyfile(self.nesProgressFile, "/tmp/progress/nes_%s.json" % value)
        except Exception, e:
            self.log.error(u'Ошибка сохранение прогресса в %s. %s' % (self.nesProgressFile, e))

    '''
    Инициализация сохранения прогресса
    '''
    def initProgress(self, messageId):
        self.progressEnable = False

        if not os.path.exists("/linuxcash/cash/exchangesystems/progress"):
            return

        self.nextStep = 0
        self.currentObjectsCount = 0
        self.saveProgress(1)
        self.countProgressFile = "/linuxcash/cash/exchangesystems/progress/counter_%s.json" % messageId

        if self.testProgressMode:
            if not os.path.exists("/tmp/progress"):
                os.makedirs("/tmp/progress")
            shutil.copyfile(self.countProgressFile, "/tmp/progress/counter.json")

        try:
            with open(self.countProgressFile, 'r') as outfile:
                data = json.load(outfile)
                self.log.info(u'Количество объектов для загрузки %s' % data['objectsCount'])
                self.progressStep = float(data['objectsCount']) / 20
                self.nextStep = self.progressStep
                self.progressEnable = True
        except Exception, e:
            self.log.error(u'Ошибка получения кол-ва объектов из %s. %s' % (self.countProgressFile, e))

    '''
    Удаляет файлы прогресса
    '''
    def finishProgress(self):
        if not self.progressEnable:
            return

        try:
            os.remove(self.countProgressFile)
        except Exception, e:
            self.log.error(u'Ошибка удаления %s. %s' % (self.countProgressFile, e))

        try:
            os.remove(self.nesProgressFile)
        except Exception, e:
            self.log.error(u'Ошибка удаления %s. %s' % (self.nesProgressFile, e))
        
        
        
    