# -*- coding: UTF-8 -*-

import httplib2
import json
import urllib
import base64
import logger
import traceback
import sys
import ConfigParser

STATUS_ERROR = 'STATUS_ERROR'                       # Ошибка загрузки/выгрузки справочника
STATUS_READY = 'STATUS_READY'                       # Справочник успешно загружен/выгружен
STATUS_READY_WITH_ERROR = 'STATUS_READY_WITH_ERROR' # Справочник загружен с ошибками
STATUS_DURING = 'STATUS_DURING'                     # Справочник в данный момент загружается/выгружается

http = httplib2.Http()
jsonHeader = {'Content-Type':'application/json', "Accept": "application/json"}

config = ConfigParser.RawConfigParser()
pathConfig = sys.path[0] + "/../../../config/nes_load.ini"
cs_prefix='/CSrest/rest'
if not config.read(pathConfig):
    print ("Configuration file %s not found. (Logger)" % pathConfig)
    exit(1)
if config.has_option('CS', 'serverUrlPrefix'):
    cs_prefix = config.get('CS', 'serverUrlPrefix')


class LoadDictStatusSender(object):
    def __init__(self, shopCode, dictCSLoadStatusId = None, dictCashLoadStatusId = None,
                statusServer = 'localhost:8080', cashCode = None, version = None, isSendLoadDictStatus=True,defaultStatusServer='localhost:8080'):
        self.shopCode = shopCode
        self.dictCSLoadStatusId = dictCSLoadStatusId
        self.dictCashLoadStatusId = dictCashLoadStatusId
        # адрес сервера, для случая когда кассе надо проинформировать о своем прогрессе         
        self.statusServer = statusServer
        self.cashCode = cashCode
        self.log = logger.Logger().getLogger(shopCode)
        self.version = version
        self.isSendLoadDictStatus = isSendLoadDictStatus
        # адрес сервера, обычно локальный, на который отправляются статусы прогрузки
        self.defaultStatusServer = defaultStatusServer

    def sendRequest(self, url, body, method):
        self.log.debug('url = %s, method = %s' % (url, method))
        body = json.dumps(body)
        self.log.debug('body = %s' % body)
        try:
            response, content = http.request(url, method, headers=jsonHeader, body=body)
            if response.status == 200:
                return json.loads(content)
            else:
                self.log.warning('Ошибка отправки статуса загрузки справочника. %s' % content)
        except Exception, e:
            self.log.warning('Ошибка отправки статуса загрузки справочника. %s' % str(e))
        return None

    def sendDictState(self, messageEn, messageRu, status, tableName = 'all'):
        if self.dictCSLoadStatusId and self.isSendLoadDictStatus:
            urlPrefix = 'http://%s%s/dicts' % (self.defaultStatusServer,cs_prefix)
            bodyBase = {'messageEn': messageEn,  'messageRu': messageRu,  'status': status}
            self.log.debug('NES mode: CS')
            if isinstance(tableName, (set, list)):
                self.log.debug('tables: %s' % tableName)
                for t in tableName:
                    self.log.debug('send state = %s for table = %s' % (status, t))
                    url, body, method = self.dictLoadCSState(urlPrefix, bodyBase, t)
                    self.sendRequest(url, body, method)
            else:
                self.log.debug('send state = %s for table = %s' % (status, tableName))
                url, body, method = self.dictLoadCSState(urlPrefix, bodyBase, tableName)
                self.sendRequest(url, body, method)

        if self.dictCashLoadStatusId:
            bodyBase = {'messageEn': messageEn,  'messageRu': messageRu,  'status': status}
            urlPrefix = 'http://%s%s/dicts' % (self.statusServer,cs_prefix)
            self.log.debug('NES mode: cash')
            url, body, method = self.dictLoadCashState(urlPrefix, bodyBase)
            self.sendRequest(url, body, method)

    def dictLoadCashState(self, urlPrefix, bodyBase):
        dictCashLoadStatusId = self.dictCashLoadStatusId
        if self.version >= '3':
            dictCashLoadStatusId += '?useDictGenerateId=true'
        url = '%s/cashLoad/%s/%s/%s' % (urlPrefix, self.shopCode, self.cashCode, dictCashLoadStatusId)
        return url, bodyBase, 'PUT'

    def dictLoadCSState(self, urlPrefix, bodyBase, tableName):
        url = '%s/CSLoad/%s' % (urlPrefix, self.shopCode)
        if tableName:
                url = url + '?tableName=%s' % tableName
        bodyBase['dictBackOffice'] = {'id': self.dictCSLoadStatusId}
        return url, bodyBase, 'POST'

# запуск генерации справочника на КС
def generateDict(shopCode, dictBackOfficeIds, owner, generateList={},statusServer = 'localhost:8080'):
    log = logger.Logger().getLogger(shopCode)
    if dictBackOfficeIds:
        for dictBackOfficeId in dictBackOfficeIds:
            url = 'http://%s%s/dicts/CSLoad/%s' % (statusServer,cs_prefix,dictBackOfficeId)
            log.debug('generate dict url=%s'%url)
            response=False
            content='Empty'
            try:
            	response, content = http.request(url, 'GET', headers=jsonHeader)
            except Exception, e:
                log.warning('Ошибка %s' % e)
                log.error(traceback.format_exc())
            	
            log.info('response=%s'%response)
            if response and response.status == 200:
                dictCSLoads = json.loads(content)
                for dictCSLoad in dictCSLoads:
                    unloadElementName = dictCSLoad.get('unloadElement', {}).get('name', None)
                    if unloadElementName:
                        if generateList.get('increment', {}).has_key(unloadElementName):
                            generateList['increment'][unloadElementName].append('')
                        else:
                            if not generateList.has_key('increment'):
                                generateList['increment'] = {}
                            generateList['increment'][unloadElementName] = ['']
            else:
                log.warning('Ошибка получения элементов выгрузки для запуска генерации справочника. %s' % content)
    generateStates = []
    for generateMode in ['increment', 'full']:
        for unloadElementName in generateList.get(generateMode, {}).keys():
            body = json.dumps({'dictId': unloadElementName, 'full': generateMode == 'full'})
            log.debug('body = %s' % body)
            cashIds = set(generateList[generateMode][unloadElementName])
            for cashId in cashIds:
                if cashId:
                    url = 'http://%s%s/tree/elements/%s/dicts?owner=%s' % (statusServer,cs_prefix,cashId, owner)
                # если идентификатор кассы не определен, то запускаем генерацию на магазин
                else:
                    url = 'http://%s%s/tree/elements/%s/dicts?owner=%s' % (statusServer,cs_prefix,shopCode, owner)
                log.debug('URL = %s' % (url))
                try:
                    response, content = http.request(url, 'POST', headers=jsonHeader, body=body)
                    if response.status == 200:
                        generateStates.append(json.loads(content))
                    else:
                        log.warning('Ошибка запуска генерации справочника. %s' % content)
                except Exception, e:
                    log.warning('Ошибка запуска генерации справочника. %s' % e)
    return generateStates if len(generateStates)>0 else None

def getQueueNames(host, brokerRestPort, user, password):
    header = jsonHeader.copy()
    header["Authorization"] = "Basic {0}".format(base64.b64encode("{0}:{1}".format(user, password)))
    url = 'http://%s:%s/rest/queue' % (host, brokerRestPort)
    try:
        response, content = http.request(url, 'GET', headers=header)
        if response.status == 200:
            return [str(queue.get('name', '')) for queue in json.loads(content)]
    except Exception, e:
        log = logger.Logger().getLogger('Service')
        log.error('Can not get queue names by REST. url = %s' % url)
    return []

def sendCompareTableResult(compareResultServerId, shopCode, cashCode, result, statusServer = 'localhost:8080'):
    log = logger.Logger().getLogger(shopCode)
    body = {'compareResultServer': {'id': compareResultServerId}, 'cashId':cashCode, 'result': result}
    url = 'http://%s%s/compareResults' % (statusServer,cs_prefix)
    body = json.dumps(body)
    log.debug('URL = %s' % (url))
    log.debug('body = %s' % unicode(body, 'unicode-escape'))
    try:
        response, content = http.request(url, 'PUT', headers=jsonHeader, body=body)
        if response.status == 200:
            pass
            # generateStates.append(json.loads(content))
        else:
            log.warning('Ошибка добавления результата для сверки таблиц. %s' % content)
    except Exception, e:
        log.warning('Ошибка добавления результата для сверки таблиц. %s' % e)

