'''
Created on 13.08.2010

@author: gsu
'''

import simplesql as sql

tables = ['valut', 'payforbiddance']

def deptsForbiddance(valut):
    return valut.get("deptsforbiddance", [])

def compileStatus(stat):
    result = 0
    result |= 1 if stat.get('basic', False) else 0
    result |= 2 if stat.get('national', False) else 0
    result |= 4 if stat.get('additional', False) else 0
    return result

roundModes = {"roundToUp": 0,
              "roundMathematical":1,
              "roundMathematicalWithLoss":2,
              "roundToDown":3,
              "roundToDownWithLoss":4}

def compileRound(round):
    return roundModes.get(round, 0)

modes = ['denypartialsum', 'denyextrasum', 'denymakechange', 'denypartialout',
         'denyinput', 'opendrawer', 'dropchange', 'denyprice', 'denypaymentforback',
         'denypaymentforsale', 'printcopyrequired', 'denypaymentanydeptforbidded']

operations = {"none": 0, "bonus": 1, "bank": 2, "certificate": 3, "processing": 10}

def compileMode(mode):

    if not mode:
        return 0

    result = 0
    for pow, item in enumerate(modes):
        result |= (2 ** pow) if mode.get(item, False) else 0
    return result

def compileOperation(op):
    return operations.get(op, 0)

def insertValut(valut):
    code = valut.get('currencycode', None)
    if code:
        yield sql.Insert(table='valut', values={'code': valut.get('currencycode', None),
                                            'stat': compileStatus(valut.get('status', {})),
                                            'chr': valut.get('shortname', ''),
                                            'name': valut.get('fullname', ''),
                                            'type': valut.get('type', 0),
                                            'rate': valut.get('rate', 1),
                                            'ratecb': valut.get('ratecb', 1),
                                            'decpoint': valut.get('decpoint', 0),
                                            'round': compileRound(valut.get('round', None)),
                                            'mode': compileMode(valut.get('mode', None)),
                                            'drawer': 1 if valut.get('drawer', False) else 0,
                                            'discascheme': 0,
                                            'ishidden': 1 if valut.get('hidden', False) else 0,
                                            'operation': compileOperation(valut.get('operation', 'none')),
                                            'payprocmodule': valut.get('payprocmodule', None),
                                            'merchantid': valut.get('merchantid', None)})
        deptForbiddances = deptsForbiddance(valut)
        yield sql.DeleteNotNestedItems(table='payforbiddance', mainCodeField='valutcode', mainCodeValue=code, nestedCodeField='deptcode',
                                       nestedCodeIn=deptForbiddances)
        for dept in deptForbiddances:
            yield sql.Insert(table='payforbiddance', values={'valutcode': code,
                                                         'deptcode': dept})

def deleteValut(valut):
    code = valut.get('currencycode', None)
    if code:
        yield sql.Delete(table='valut', whereFields=['code'], whereValues=[code])
        yield sql.Delete(table='payforbiddance', whereFields=['valutcode'], whereValues=[code])

def deleteValutByCode(valutcode):
    yield sql.Delete(table='valut', whereFields=['code'], whereValues=[valutcode])
    yield sql.Delete(table='payforbiddance', whereFields=['valutcode'], whereValues=[valutcode])

def clearValut():
    yield sql.DeleteAll(table='valut')
    yield sql.DeleteAll(table='payforbiddance')
