# -*- coding: UTF-8 -*-

import sys, os
from qpid.messaging import Connection, Empty, NotFound
import ConfigParser, time
import glob
import marshal
import atexit
import tempfile
import traceback
import threading
import json
import db_inserter
import db_request_executor
from bottle import route, run, response, request
from httplib import INTERNAL_SERVER_ERROR
from nes_common import logger, rest_client
from nes_common.basictaskrunner import BasicTaskRunner
from nes_common.mproctaskrunner import MprocTaskRunner
from _mysql_exceptions import OperationalError, DatabaseError

serviceLog = logger.Logger().getLogger('Service')

lockFile = os.path.normpath('%s/locks/lockQueueGetter' % tempfile.gettempdir())
prefixQueueName = 'artix.loaddict.'
taskRunner = None
timeShopsReread = 0

def unlock():
    if os.path.exists(lockFile):
        serviceLog.info('remove lock file %s' % lockFile)
        os.remove(lockFile)

atexit.register(unlock)

def getShopDests(host, port, user, password, brokerType='cpp', brokerRestPort=None):
    serviceLog.debug('Получаем список магазинов...')
    shopDests = []
    if brokerType == 'java':
        queueNames = rest_client.getQueueNames(host, brokerRestPort, user, password)
        for name in queueNames:
            if name.startswith(prefixQueueName):
                shopDests.append(name.replace(prefixQueueName, ''))
    else:
        broker = None
        connection = None
        try:
            from qpidtoollibs import BrokerAgent
            connection = Connection.establish('%s:%s' % (host, port))
            broker = BrokerAgent(connection)
            queues = broker.getAllQueues()
            for q in queues:
                if q.name.startswith(prefixQueueName):
                    shopDests.append(q.name.replace(prefixQueueName, ''))
        except Exception, e:
            serviceLog.warning(u'Ошибка получения списка магазинов из очередей. %s' % str(e))
            serviceLog.warning(traceback.format_exc())
        finally:
            if broker:
                broker.close()
            if connection:
                connection.close()
    serviceLog.debug('shopDests = %s' % shopDests)
    return shopDests

@route('/nes/queue_reread', method='PUT')
def restQueueReread():
    global timeShopsReread
    response.content_type = 'application/json'
    result = {u'success': False}
    try:
        timeShopsReread = 0
        serviceLog.info(u'Change timeShopsReread =  %s' % timeShopsReread)
        result = {u'success': True}
    except Exception, e:
        serviceLog.warning(u'Ошибка REST сервиса. %s' % str(e))
        serviceLog.warning(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False}
    return json.dumps(result, ensure_ascii=False)

@route('/nes/queue_load', method='PUT')
def restLoad():
    body = json.loads(request.body.read())
    shopId = str(body.get('shopId'))
    response.content_type = 'application/json'
    result = {u'success': False}
    try:
        serviceLog.info('Put new task for load shop:%s' % shopId)
        taskRunner.addAsyncTask(shopId, receiveMessages, [shopId, brokerServer, defaultNesGenerateCommandMode])
        result = {u'success': True}
    except Exception, e:
#         serviceLog.warning(u'Ошибка REST сервиса. %s' % str(e))
        serviceLog.warning(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False}
    return json.dumps(result, ensure_ascii=False)

@route('/nes/forgot_bad', method='PUT')
def restForgot():
    global taskRunner
    body = json.loads(request.body.read(), encoding='utf8')
    shopId = body.get('shopId')
    response.content_type = 'application/json'
    result = {u'success': False}
    try:
        serviceLog.info('Forgot bad task for shop:%s' % shopId)
        taskRunner.forgotBadTasks()
        result = {u'success': True}

    except Exception, e:
        serviceLog.warning(u'Ошибка REST сервиса. %s' % str(e))
        serviceLog.warning(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False}
    return json.dumps(result, ensure_ascii=False)

def dictsCountCommandToStr(countGoodCommands, countBadCommands, command, ruName):
    if countGoodCommands[command] + countBadCommands[command] > 0:
        s_ru = 'Команды %s: успешно - %s, ошибок - %s; ' % (ruName, countGoodCommands[command], countBadCommands[command])
        s_en = '%s commans: ok - %s, error - %s; ' % (command, countGoodCommands[command], countBadCommands[command])
        return s_ru, s_en
    return '', ''

def dictsCountCommandsToStr(countGoodCommands, countBadCommands):
    clear_ru, clear_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'clear', 'очистки')
    add_ru, add_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'add', 'добавления')
    delete_ru, delete_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'delete', 'удаления')
    update_ru, update_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'update', 'обновления')
    return clear_ru + add_ru + delete_ru + update_ru, clear_en + add_en + delete_en + update_en

def openConnection(connection, success):
    connection.open(timeout=5)
    success["successFlag"] = True

def receiveMessages(shopDest, broker, defaultNesGenerateCommandMode):
    shop = {'shopId': shopDest.split('.')[0]}
    shop['clusterId'] = shopDest.replace(shop['shopId'] + '.', '') if shop['shopId'] != shopDest else ''
    log = logger.Logger().getLogger(shop['shopId'])
    log.debug('Connect to broker = %s' % broker)
    try:
        connection = Connection(url=broker)
        if newLib:
            # Откуда вырасли такие сложности: по какой-то причине иногда поток провисает на открытии соединения с qpid, и после этого может заморозиться прогрузка
            # справочников на подвисший магазин. Чтоб такое провисание не влияло на загрузку - открытие соединения помещено в отдельный поток. Даже если этот доп-
            # поток провиснет, основной продолжит свою работу и
            # следующим запросом прогрузка на магазин продолжится. Что при этом происходит с коннекшенами с qpid - вопрос открытый.
            # Как проявляется ошибка: нужно чтоб примерно одновременно запускался сервис qpid и nes. 1 раз из 10 - помещаемые в очередь loaddict справочники перестанут загружаться
            # Когда ошибка с подвисанием потока нивелируется - данную "оптимизацию" можно убрать.
            params = {"connection": connection, "success": {"successFlag": False}}
            openConThread = threading.Thread(target=openConnection, kwargs=params)
            openConThread.setDaemon(True)
            openConThread.setName("oConn")
            openConThread.start()
            openConThread.join(timeout=5)
            if not params["success"]["successFlag"]:
                log.error("qpid open connection timeout")
                serviceLog.error("qpid open connection timeout for code: %s", shop["shopId"])
                return True
        else:
            connection.open()
        log.debug('Successfully open connection')
    except Exception, e:
        log.error('error open connection')
        serviceLog.error(traceback.format_exc())
        return True
    try:
        session = connection.session()
        log.debug('Successfully create session')
    except Exception, e:
        serviceLog.error(traceback.format_exc())
        connection.close()
        return True
    receiver = None
    try:
        receiver = session.receiver('%s%s' % (prefixQueueName, shopDest))
        log.debug('Successfully create receiver')
    except NotFound:
        log.info('Очередь %s%s еще не создана.' % (prefixQueueName, shopDest))
        connection.close()
        return True
    dictLoadStatusId = None
    cashCode = None
    statusServer = 'localhost:8080'
    loadDictStatusSender = None
    dictBackOfficeIds = set()
    generateList = {}
    while True:
        try:
            message = receiver.fetch(0)
            if message.properties.get('type') == 'control' and message.properties.get('control') == 'begin_tran':
                log.info('Начало транзакции...')
                nesGenerateCommandMode = message.properties.get('nesGenerateCommandMode', defaultNesGenerateCommandMode)
                nesGenerateCommandCashesList = [x.strip() for x in message.properties.get('nesGenerateCommandCashesList', '').split(',')]
                unloadElementName = message.properties.get('unloadElementName', None)
                owner = message.properties.get('nesGenerateCommandOwner', '')

                dictCSStatusId = message.properties.get('dictBackOfficeId')

                isSendLoadDictStatus = True

                if nesGenerateCommandMode != 'none':
                    if unloadElementName:
                        # если пришла просто команда на генерацию справочника без самого справочника, то не будем отправлять статус загрузки в БД
                        isSendLoadDictStatus = False
                        if generateList.get(nesGenerateCommandMode, {}).has_key(unloadElementName):
                            generateList[nesGenerateCommandMode][unloadElementName] += nesGenerateCommandCashesList
                        else:
                            if not generateList.has_key(nesGenerateCommandMode):
                                generateList[nesGenerateCommandMode] = {}
                            generateList[nesGenerateCommandMode][unloadElementName] = nesGenerateCommandCashesList
                    else:
                        # если передали статус генерации на КС и режим генерации не установлен в none и не передали unloadElementName,
                        # то для этого справочника будем искать unloadElement-ы для запуска генерации
                        if dictCSStatusId:
                            dictBackOfficeIds.add(dictCSStatusId)

                dictCashStatusId = message.properties.get('dictCashLoadStateId')
                cashCode = message.properties.get('cashCode')
                statusServer = message.properties.get('statusServer')

                version = message.properties.get('version')

                loadDictStatusSender = rest_client.LoadDictStatusSender(shop['shopId'], dictCSLoadStatusId=dictCSStatusId, dictCashLoadStatusId=dictCashStatusId,
                                                                        statusServer=statusServer, cashCode=cashCode, version=version, isSendLoadDictStatus=isSendLoadDictStatus,defaultStatusServer=defaultStatusServer)
                try:
                    inserter = db_inserter.DB_Inserter(shop, loadDictStatusSender)
                except (DatabaseError, OperationalError) as e:
                    log.error('Ошибка подключения к БД. %s' % e)
                    log.info('Попробуем выполнить загрузку позже.')
                    connection.close()
                    return True
                log.info('Подготовка к загрузке справочников...')
                inserter.start(message.properties.get('startMessageId'))
                typeMessage = None
                log.info('Просматриваем сообщения...')
                loadObjects = False
                tableNames = set()
                countGoodCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
                countBadCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
                while typeMessage != 'control':
                    try:
                        message = receiver.fetch()
                    except Empty:
                        loadDictStatusSender.sendDictState('Not found message "end transaction"', 'Не найдено сообщение "конец транзакции"', rest_client.STATUS_ERROR)
                        log.error("Не найдено сообщение 'конец транзакции'")
                        connection.close()
                        return False
                    typeMessage = message.properties.get('type')
                    if typeMessage == 'datagram':
                        if message.properties.get('serialType') == 'marshal':
                            dictObjects = marshal.loads(message.content)
                            log.info('Добавляем объекты в БД')
                            try:
                                tables, countGoodCommandsInPackage, countBadCommansInPackage = inserter.add_package(dictObjects)
                                for t in tables:
                                    tableNames.add(t)
                                for key in countGoodCommands.keys():
                                    countGoodCommands[key] += countGoodCommandsInPackage[key]
                                    countBadCommands[key] += countBadCommansInPackage[key]
                            except Exception, e:
                                log.error('Ошибка при добавлении объектов на загрузку в БД: %s' % e)
                                try:
                                    inserter.flagUnBlockSet()
                                    connection.close()
                                except Exception, e:
                                    log.error('Ошибка разблокировки БД и закрытия соединения: %s' % e)
                                loadDictStatusSender.sendDictState(e.message, 'Ошибка: %s' % e.message, rest_client.STATUS_ERROR)
                                return False
                            loadObjects = True
                    elif typeMessage != 'control':
                        log.warning('Получили сообщение неизвестного формата.')
                if not loadObjects:
                    log.warning('Не получено ни одного сообщения с объектами из справочника.')
                log.info('Закончили обход объектов в текущей транзакции')
                if message.properties.get('type') == 'control' and message.properties.get('control') == 'end_tran':
                    result = inserter.stop()
                    if not result:
                        loadDictStatusSender.sendDictState('Stop failure insert into DB', 'Ошибка завершения вставки в БД', rest_client.STATUS_ERROR)
                        connection.close()
                        log.info('Попробуем выполнить загрузку позже')
                        return True
                    session.acknowledge()
                    log.info('Транзакция для магазина - %s окончена.' % shop['shopId'])
                    mes_ru, mes_en = dictsCountCommandsToStr(countGoodCommands, countBadCommands)
                    log.info(mes_ru)
                    if sum(countBadCommands.values()) > 0:
                        loadDictStatusSender.sendDictState('Dictionary is not fully loaded. ' + mes_en, 'Справочник загружен, но не полностью. Были ошибки. ' + mes_ru,
                                                           rest_client.STATUS_READY_WITH_ERROR, tableNames)
                        if not loadDictStatusSender.cashCode:  # при загрузке в КС для остальных таблиц, у которых была успешная загрузка ставим "хороший" статус
                            loadDictStatusSender.sendDictState('OK ' + mes_en, 'ОК ' + mes_ru, rest_client.STATUS_READY)
                    else:
                        loadDictStatusSender.sendDictState('OK ' + mes_en, 'ОК ' + mes_ru, rest_client.STATUS_READY)
                else:
                    log.error("Не найдено сообщение 'конец транзакции'")
                    loadDictStatusSender.sendDictState('Not found message "end transaction"', 'Не найдено сообщение "конец транзакции"', rest_client.STATUS_ERROR)
                    connection.close()
                    return False
            elif message.properties.get('type') == 'comparetable':
                request = message.properties.get('requestCash')
                compareName = message.properties.get('compareName')
                compareResultServerId = message.properties.get('compareResultServerId')
                statusServer = message.properties.get('statusServer')
                cashCode = message.properties.get('cashCode')
                log.info(u"Получено сообщение для сверки данных <%s>. request: %s" % (compareName, request))
                log.debug(u"Доп. пораметры: compareResultServerId = %s, statusServer = %s, cashId = %s" 
                          % (compareResultServerId, statusServer, cashCode))
                requestExecutor = db_request_executor.DB_RequestExecutor(shop)
                log.info("Выполняем запрос в БД")
                executeResult = requestExecutor.execute(request)
                resultServer = message.content
                if resultServer != None:
                    log.debug(u"Резальтат выполнения запроса на КС: %s" % (unicode(resultServer, 'utf-8')))
                log.debug(u"Резальтат выполнения запроса: %s" % (str(executeResult).decode('string-escape').decode('utf8')))
                log.info("Отправляем результат запроса на сервер.")
                rest_client.sendCompareTableResult(compareResultServerId, shop.get('shopId', ''), cashCode, executeResult, statusServer)
                session.acknowledge()
            else:
                log.warning('Получили сообщение неизвестного формата.')
                session.acknowledge()
        except Empty:
            log.debug('Проверка сообщений для магазина - %s закончена' % shop['shopId'])
            if dictBackOfficeIds or generateList:
                log.info('Запускаем генерацию справоников на КС')
                rest_client.generateDict(shop['shopId'], dictBackOfficeIds, owner, generateList,defaultStatusServer)
            connection.close()
            return True
        except Exception, e:
            log.error('Ошибочка: %s' % traceback.format_exc())
            connection.close()
            if loadDictStatusSender:
                loadDictStatusSender.sendDictState(str(e), 'Ошибка: %s' % e, rest_client.STATUS_ERROR)
            return False
    return True

def getIntFromConfig(config, nameOfParam, defaultValue, section='QUEUE'):
    return config.getint(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

def getStrFromConfig(config, nameOfParam, defaultValue, section='QUEUE'):
    return config.get(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

def getBooleanFromConfig(config, nameOfParam, defaultValue, section='QUEUE'):
    return config.getboolean(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

def startRestServiceForCS(config, isCash):
    if not isCash:
        restServiceHost = getStrFromConfig(config, 'host', '0.0.0.0', 'REST_SERVICE')
        restServicePort = getIntFromConfig(config, 'port', 28081, 'REST_SERVICE')
        try:
            serviceLog.debug('runing REST service on %s:%s...' % (restServiceHost, restServicePort))
            t = threading.Thread(target=run, kwargs={'host':restServiceHost, 'port':restServicePort})
            t.daemon = True
            t.start()
        except Exception, e:
            serviceLog.warning(u'Ошибка запуска REST сервиса: %s' % e.message)
            serviceLog.warning(traceback.format_exc())

def start(service=True, isCash=True):
    global taskRunner
    global timeShopsReread
    global brokerServer
    global defaultNesGenerateCommandMode
    global newLib
    global defaultStatusServer

    taskRunner = BasicTaskRunner() if isCash else MprocTaskRunner()
    # в lucid устаревшая версия библиотеки для работы с qpid, часть параметров не действуют, нужно организовать свое поведение
    newLib = not isCash
    if service and os.path.exists(lockFile):
        serviceLog.debug('Exist lock file %s' % lockFile)
        serviceLog.error("Can not run service. This service is runing.")
        os._exit(1)  # выход без выполнения atexit
    config = ConfigParser.RawConfigParser()
    if not config.read(sys.path[0] + "/../../../config/nes_load.ini"):
        serviceLog.error("Configuration file 'nes_load.ini' not found.")
        sys.exit(1)
    poolProcessSize = config.getint('MAIN', 'poolProcessSize')
    host = getStrFromConfig(config, 'brokerHost', 'localhost')
    port = getIntFromConfig(config, 'brokerPort', 5672)
    user = getStrFromConfig(config, 'user', 'guest')
    password = getStrFromConfig(config, 'password', 'guest')
    brokerType = getStrFromConfig(config, 'brokerType', 'cpp')
    brokerRestPort = getIntFromConfig(config, 'brokerRestPort', 9080)
    sendDictGenerateEventOnCS = getBooleanFromConfig(config, 'sendDictGenerateEventOnCS', True, 'MAIN')

    defaultNesGenerateCommandMode = 'increment' if sendDictGenerateEventOnCS else 'none'

    defaultStatusServer = getStrFromConfig(config,'defaultStatusServer', 'localhost:8080','CS')


    broker = '%s/%s@%s:%s' % (user, password, host, port)
    brokerServer = broker
    nesProgressFile = "/linuxcash/cash/exchangesystems/progress/nes.json"
    countProgressFile = "/linuxcash/cash/exchangesystems/progress/counter_*.json"
    serviceLog.debug('service mode = %s' % service)
    serviceLog.debug('isCash = %s' % isCash)
    if service:
        startRestServiceForCS(config, isCash)
        if not os.path.exists(os.path.dirname(lockFile)):
            os.makedirs(os.path.dirname(lockFile))
        f = open(lockFile, 'w')
        f.close()


    results = {}
    shopDests = []
    taskRunner.init(poolProcessSize)
    serviceLog.debug('Start monitoring...')
    forever = True
    while forever:
        forever = service

        if isCash:
            shopDests = ['cash']
            connection = Connection(broker)
            connection.open()
            session = connection.session()
            try:
                receiver = session.receiver('%s%s' % (prefixQueueName, 'cash'))
                message = receiver.fetch(0)
            except (Empty, NotFound):
                try:
                    if os.path.exists(nesProgressFile):
                        os.remove(nesProgressFile)
                except Exception, e:
                    serviceLog.error(u'Ошибка удаления %s. %s' % (nesProgressFile, e))

                try:
                    for file in glob.glob(countProgressFile):
                        os.remove(file)
                except Exception, e:
                    serviceLog.error(u'Ошибка удаления %s. %s' % (countProgressFile, e))
            finally:
                connection.close()
        else:
            if timeShopsReread == 0 or (time.time() - 600) > timeShopsReread:
                timeShopsReread = time.time()
                tmpShopDests = getShopDests(host, port, user, password, brokerType, brokerRestPort)
                if tmpShopDests:
                    shopDests = tmpShopDests


        for shopDest in shopDests:
            taskRunner.addAsyncTask(shopDest, receiveMessages, [shopDest, broker, defaultNesGenerateCommandMode])
        if service:
            time.sleep(20)

def stop():
    serviceLog.info('Stoping...')
    if taskRunner:
        taskRunner.stopAllTasks()
    serviceLog.info('Exit...')
    exit()
